/*
** Module   : P2JQuery.java
** Abstract : Defines common API for all legacy query implementations
**
** Copyright (c) 2004-2013, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- --JPRM-- ------------------Description-------------------
** 001 ECF 20060102   @23792 Created initial version. Defines API common
**                           to all query implementations.
** 002 ECF 20060501   @25916 Added isPreselect method. Indicates query's
**                           preselection behavior.
** 003 ECF 20060504   @25971 Set superinterface to WrappedResource. Added
**                           addAccumulator variants.
** 004 ECF 20061023   @30606 Added setErrorIfNull() method. Previously,
**                           this method existed in various implementors
**                           of this interface, but it is required from
**                           the UI (BrowseWidget, specifically), so it
**                           was moved to this interface.
** 005 ECF 20061102   @30958 Added support for reposition notification and
**                           auto-fetch features.
** 006 ECF 20061204   @31575 Added methods to support browse.
** 007 SIY 20070502   @33427 Added notifyRepositionListeners() method.
** 008 ECF 20070716   @34509 Added methods:
**                              isStandalone()
**                              setStandalone(boolean)
**                              cleanup()
** 009 ECF 20080509   @38242 Added open() method. Necessary for converted
**                           OPEN QUERY statements.
** 010 SVL 20080604   @38530 currentRow() renamed to currentRowImpl().
**                           Added new currentRow() function as Progress-
**                           compatible version.
** 011 ECF 20080729   @39320 Added rowid support. Refactored reposition*()
**                           methods.
** 012 ECF 20081106   @40323 Added _isOffEnd(). Returns a boolean instead
**                           of a logical, primarily for internal, runtime
**                           use.
** 013 CA  20081210   @40838 Implemented CLOSE QUERY statement.
** 014 ECF 20090312   @42984 Added getTableCount(). Moved up from Joinable
**                           interface to support NUM-BUFFERS attribute on a
**                           query handle.
** 015 CA  20090731   @43467 Added getOffEndListeners, which returns a list of
**                           all off-end listeners for all used buffers.
** 016 SVL 20091030   @44294 Added getOffEnd() function.
** 017 VMN 20130127          Added support for handle-based methods:
**                           GET-BUFFER-HANDLE (KW_GET_BUFH)
**                           GET-CURRENT (KW_GET_CUR),
**                           QUERY-CLOSE (KW_QRY_CLOS),
**                           REPOSITION-FORWARD (KW_REPOS_F),
**                           REPOSITION-TO-ROW (KW_REPOS_2R)
**                           REPOSITION-TO-ROWID (KW_REPOS_2I)
**                           and attributes
**                           NUM-RESULTS (KW_NUM_RES)
** 018 OM  20130114          Added declarations for SET-BUFFERS(KW_SET_BUF), QUERY-PREPARE(KW_QRY_PREP),
**                           and QUERY-OPEN(KW_QRY_OPEN).
** 019 VMN 20130213          Added support for attributes
**                           INDEX-INFORMATION (KW_IDX_INFO)
**                           FORWARD-ONLY (KW_FWD_ONLY)
**                           PREPARE_STRING (KW_PREP_STR)
** 020 VMN 20130219          Added support for attributes
**                           IS-OPEN (KW_IS_OPEN)
** 021 CA  20130221          Added ADM-DATa, UNIQUE-ID and CommonHandleChain support.
** 022 VMN 20130306          Corrected support for handle-based methods:
**                           GET-NEXT (KW_GET_NEXT)
**                           GET-PREV (KW_GET_PREV)
**                           GET-FIRST (KW_GET_1ST)
**                           GET-LAST (KW_GET_LAST)
**                           GET-CURRENT (KW_GET_CUR)
**                           Added addBuffer(handle bufHandle)
** 023 CA  20130311          Added bufferHandle(integer).
** 024 VMN 20130311          Cosmetics (changed parameter names and comments)
*/

package com.goldencode.p2j.persist;

import java.util.List;

import com.goldencode.p2j.persist.event.*;
import com.goldencode.p2j.util.*;

/**
 * The minimal API all query implementations must provide.  Defines methods
 * to retrieve and navigate results and to access query metadata.
 */ 
public interface P2JQuery
extends WrappedResource,
        ADMData,
        UniqueID,
        CommonHandleChain
{
   /**
    * Enable results list scrolling/repositioning.
    */
   public void setScrolling();
   
   /**
    * Open the prepared query.
    */
   public void open();
   
   /**
    * Explicitly close the prepared query.
    */
   public void close();

   /**
    * Get the number of tables joined by this query.
    *
    * @return  Number of joined tables.
    */
   public int getTableCount();
   
   /**
    * Get all the off-end listeners associated with this query.
    * 
    * @return  the off-end listeners associated with this query.
    */
   public List<QueryOffEndListener> getOffEndListeners();
   
   /**
    * Set a mode such that setting a null record into a backing buffer may or
    * may not result in an error condition.  This applies only to requests to
    * retrieve <code>FIRST</code>, <code>LAST</code>, and <code>UNIQUE</code>
    * records.
    *
    * @param   errorIfNull
    *          If <code>true</code>, setting a <code>null</code> value as the
    *          current record in an underlying buffer will raise an error
    *          condition (if not in silent error mode); if <code>false</code>,
    *          this action will raise an end condition instead.
    */
   public void setErrorIfNull(boolean errorIfNull);
   
   /**
    * Add an accumulator to this query.  The accumulator expects to be
    * notified whenever a new result row is accessed (during sequential row
    * access).
    * <p>
    * This implementation assumes the accumulator is not associated with a
    * break group.
    *
    * @param   accum
    *          Accumulator to be registered with this query.
    */
   public void addAccumulator(Accumulator accum);
   
   /**
    * Add an accumulator to this query.  The accumulator expects to be
    * notified whenever a new result row is accessed (during sequential row
    * access).
    * <p>
    * This implementation assumes the accumulator is not associated with a
    * break group.
    *
    * @param   accum
    *          Accumulator to be registered with this query.
    * @param   deferred
    *          <code>true</code> to defer accumulator recalculation until the
    *          next invocation of {@link
    *          com.goldencode.p2j.util.Accumulator#accumulate()}.  If
    *          <code>false</code>, the accumulator recalculates its internal
    *          state as soon as the query's next result is fetched.
    */
   public void addAccumulator(Accumulator accum, boolean deferred);
   
   /**
    * Retrieve the first composite row of results for the query.  The
    * underlying buffer for each query component is updated with the
    * appropriate record.
    * <p>
    * If the query involves outer joins, some of the underlying buffers may
    * be empty when this method returns.
    *
    * @throws  ErrorConditionException
    *          if the query or the fetch of any record fails, or if no records
    *          are available and the query is set to fail in this case.
    * @throws  QueryOffEndException
    *          if no records are available and the query is set not to fail
    *          in this case.
    */
   public void first();
   
   /**
    * Retrieve the first composite row of results for the query.  The
    * underlying buffer for each query component is updated with the
    * appropriate record.
    * <p>
    * If the query involves outer joins, some of the underlying buffers may
    * be empty when this method returns.
    *
    * @param   lockType
    *          Lock type to apply to records retrieved (overrides default
    *          lock type set for each query component).
    *
    * @throws  ErrorConditionException
    *          if the query or the fetch of any record fails, or if no records
    *          are available and the query is set to fail in this case.
    * @throws  QueryOffEndException
    *          if no records are available and the query is set not to fail
    *          in this case.
    */
   public void first(LockType lockType);
   
   /**
    * Retrieve the last composite row of results for the query.  The
    * underlying buffer for each query component is updated with the
    * appropriate record.
    * <p>
    * If the query involves outer joins, some of the underlying buffers may
    * be empty when this method returns.
    *
    * @throws  ErrorConditionException
    *          if the query or the fetch of any record fails, or if no records
    *          are available and the query is set to fail in this case.
    * @throws  QueryOffEndException
    *          if no records are available and the query is set not to fail
    *          in this case.
    */
   public void last();
   
   /**
    * Retrieve the last composite row of results for the query.  The
    * underlying buffer for each query component is updated with the
    * appropriate record.
    * <p>
    * If the query involves outer joins, some of the underlying buffers may
    * be empty when this method returns.
    *
    * @param   lockType
    *          Lock type to apply to records retrieved (overrides default
    *          lock type set for each query component).
    *
    * @throws  ErrorConditionException
    *          if the query or the fetch of any record fails, or if no records
    *          are available and the query is set to fail in this case.
    * @throws  QueryOffEndException
    *          if no records are available and the query is set not to fail
    *          in this case.
    */
   public void last(LockType lockType);
   
   /**
    * Retrieve the next composite row of results for the query.  The
    * underlying buffer for each query component is updated with the
    * appropriate record.
    * <p>
    * If the query involves outer joins, some of the underlying buffers may
    * be empty when this method returns.
    *
    * @throws  ErrorConditionException
    *          if the query or the fetch of any record fails, or if no records
    *          are available and the query is set to fail in this case.
    * @throws  QueryOffEndException
    *          if no records are available and the query is set not to fail
    *          in this case.
    */
   public void next();
   
   /**
    * Retrieve the next composite row of results for the query.  The
    * underlying buffer for each query component is updated with the
    * appropriate record.
    * <p>
    * If the query involves outer joins, some of the underlying buffers may
    * be empty when this method returns.
    *
    * @param   lockType
    *          Lock type to apply to records retrieved (overrides default
    *          lock type set for each query component).
    *
    * @throws  ErrorConditionException
    *          if the query or the fetch of any record fails, or if no records
    *          are available and the query is set to fail in this case.
    * @throws  QueryOffEndException
    *          if no records are available and the query is set not to fail
    *          in this case.
    */
   public void next(LockType lockType);
   
   /**
    * Retrieve the previous composite row of results for the query.  The
    * underlying buffer for each query component is updated with the
    * appropriate record.
    * <p>
    * If the query involves outer joins, some of the underlying buffers may
    * be empty when this method returns.
    *
    * @throws  ErrorConditionException
    *          if the query or the fetch of any record fails, or if no records
    *          are available and the query is set to fail in this case.
    * @throws  QueryOffEndException
    *          if no records are available and the query is set not to fail
    *          in this case.
    */
   public void previous();
   
   /**
    * Retrieve the previous composite row of results for the query.  The
    * underlying buffer for each query component is updated with the
    * appropriate record.
    * <p>
    * If the query involves outer joins, some of the underlying buffers may
    * be empty when this method returns.
    *
    * @param   lockType
    *          Lock type to apply to records retrieved (overrides default
    *          lock type set for each query component).
    *
    * @throws  ErrorConditionException
    *          if the query or the fetch of any record fails, or if no records
    *          are available and the query is set to fail in this case.
    * @throws  QueryOffEndException
    *          if no records are available and the query is set not to fail
    *          in this case.
    */
   public void previous(LockType lockType);
   
   /**
    * Retrieve a unique record (only valid for a single-table query).
    *
    * @throws  ErrorConditionException
    *          if the query fails or if more than one result is found which
    *          matches the query criteria.
    */
   public void unique();
   
   /**
    * Retrieve a unique record (only valid for a single-table query),
    * overriding the default lock type of the underlying query component.
    *
    * @param   lockType
    *          Lock type which should override that of the query component.
    *
    * @throws  ErrorConditionException
    *          if the query fails or if more than one result is found which
    *          matches the query criteria.
    */
   public void unique(LockType lockType);
   
   /**
    * Reload the current composite row of results for the query.
    * <p>
    * If the query involves outer joins, some of the underlying buffers may
    * be empty when this method returns.
    *
    * @throws  ErrorConditionException
    *          if the reload triggers an error.
    */
   public void current();
   
   /**
    * Reload the current composite row of results for the query, overriding
    * the lock type to apply to each record.
    * <p>
    * If the query involves outer joins, some of the underlying buffers may
    * be empty when this method returns.
    *
    * @param   lockType
    *          Lock type which should override that of any query component.
    *
    * @throws  ErrorConditionException
    *          if the reload triggers an error.
    */
   public void current(LockType lockType);
   
   /**
    * Reposition the cursor such that a request to retrieve the next result
    * will retrieve the result which matches the specified primary key ID.  A
    * request to retrieve the previous result will retrieve the result
    * immediately previous to this, if any, in the results list.
    *
    * @param   id
    *          A primary key ID, representing the target record (in the case
    *          of a join, this ID is associated with the left-most record in
    *          the join).
    * 
    * @throws  ErrorConditionException
    *          if the specified ID represents the unknown value.
    */
   public void repositionByID(recid id);
   
   /**
    * Reposition the cursor such that a request to retrieve the next result
    * will retrieve the result which matches the specified array of primary
    * key IDs.  A request to retrieve the previous result will retrieve the
    * result immediately previous to this, if any, in the results list.
    *
    * @param   id1
    *          A primary key ID, representing the target record (in the case
    *          of a join, this ID is associated with the left-most record in
    *          the join).
    * @param   joinIDs
    *          All remaining primary key IDs, if any, arranged from left to
    *          right to coincide with the records being joined by the
    *          underlying query.
    * 
    * @throws  ErrorConditionException
    *          if any of the specified IDs represents the unknown value.
    */
   public void repositionByID(rowid id1, rowid...joinIDs);
   
   /**
    * Reposition the cursor to the specified row in the result set.  The row
    * indicates the position between two results, such that a call to
    * retrieve the next record will get the result at the specified row,
    * and a call to retrieve the previous result will get the result before
    * the new position.
    *
    * @param   row
    *          1-based index of the target position.
    *
    * @throws  ErrorConditionException
    *          if <code>row</code> represents unknown value.
    */
   public void reposition(NumberType row);
   
   /**
    * Reposition the query to the specified row in the result set.  The row
    * indicates the position between two results, such that a call to
    * retrieve the next record will get the record after the new position,
    * and a call to retrieve the previous record will get the record before
    * the new position.
    *
    * @param   row
    *          1-based index of the target position.
    */
   public void reposition(int row);
   
   /**
    * Advance the current cursor position forward by the specified number of
    * rows.  This will always leave the cursor between two results.  For
    * example, given a request to move 1 row forward:
    * <ul>
    *   <li>if the cursor currently is positioned <i>directly on a result</i>,
    *       this would move it ahead <i>one and a half positions</i>, to just
    *       beyond the following result;
    *   <li>if the cursor currently is positioned <i>between two results</i>,
    *       this would move it ahead <i>one position</i>, to just beyond the
    *       following result.
    * </ul>
    *
    * @param   rows
    *          Number of rows to scroll the cursor forward.
    *
    * @throws  ErrorConditionException
    *          if <code>rows</code> represents the unknown value.
    */
   public void forward(NumberType rows);
   
   /**
    * Advance the current cursor position forward by the specified number of
    * rows.  This will always leave the cursor between two results.  For
    * example, given a request to move 1 row forward:
    * <ul>
    *   <li>if the cursor currently is positioned <i>directly on a result</i>,
    *       this would move it ahead <i>one and a half positions</i>, to just
    *       beyond the following result;
    *   <li>if the cursor currently is positioned <i>between two results</i>,
    *       this would move it ahead <i>one position</i>, to just beyond the
    *       following result.
    * </ul>
    *
    * @param   rows
    *          Number of rows to scroll the cursor forward.
    */
   public void forward(int rows);
   
   /**
    * Move the current cursor position backward by the specified number of
    * rows.  This will always leave the cursor between two results.  For
    * example, given a request to move 1 row backward:
    * <ul>
    *   <li>if the cursor currently is positioned <i>directly on a result</i>,
    *       this would move it back <i>one half position</i>, to just
    *       before the current result;
    *   <li>if the cursor currently is positioned <i>between two results</i>,
    *       this would move it back <i>one position</i>, to just before the
    *       current result.
    * </ul>
    *
    * @param   rows
    *          Number of rows to scroll the cursor backward.
    *
    * @throws  ErrorConditionException
    *          if <code>rows</code> represents the unknown value.
    */
   public void backward(NumberType rows);
   
   /**
    * Move the current cursor position backward by the specified number of
    * rows.  This will always leave the cursor between two results.  For
    * example, given a request to move 1 row backward:
    * <ul>
    *   <li>if the cursor currently is positioned <i>directly on a result</i>,
    *       this would move it back <i>one half position</i>, to just
    *       before the current result;
    *   <li>if the cursor currently is positioned <i>between two results</i>,
    *       this would move it back <i>one position</i>, to just before the
    *       current result.
    * </ul>
    *
    * @param   rows
    *          Number of rows to scroll the cursor backward.
    */
   public void backward(int rows);
   
   /**
    * Get the <b><i>current</i></b> size of the cached results list.  However,
    * since this cache may be built progressively as the query is scrolled,
    * repositioned, and naturally navigated, this <b>does not</b> necessarily
    * represent the full count of results which may satisfy the query.
    *
    * @return  Current results list size or unknown value if the query is not
    *          open.
    */
   public integer size();
   
   /**
    * Progress-compatible function that returns the 1-based index of the
    * current row in the result set. This is the index of the entry at or
    * before which the cursor currently is positioned.
    *
    * @return  Unknown value if the query is not open or result set is empty.
    *          1 if the query is positioned before the first record.
    *          A value 1 greater than the number of rows in the query result
    *          set if the query is positioned beyond the last record.
    *          Otherwise returns the number of rows in the query result set. 
    *
    */
   public integer currentRow();
   
   /**
    * Get the 1-based index of the current row in the result set.  This is
    * the index of the entry at or before which the cursor currently is
    * positioned.
    *
    * @return  Current row or the unknown value if the current row cannot be
    *          determined.  The current row cannot be determined if the
    *          results list is empty, or if the cursor currently is in
    *          backward scroll mode.
    */
   public integer currentRowImpl();
   
   /**
    * Indicate whether the cursor has run off one or the other end of the
    * associated query's list of available results.  If <code>true</code>,
    * this indicates that the query cannot produce any more results in the
    * current scroll direction.
    *
    * @return  <code>true</code> if the cursor is off either end of its
    *          query's results list.
    */
   public logical isOffEnd();
   
   /**
    * Indicate whether the cursor has run off one or the other end of the
    * associated query's list of available results.  If <code>true</code>,
    * this indicates that the query cannot produce any more results in the
    * current scroll direction.
    *
    * @return  <code>true</code> if the cursor is off either end of its
    *          query's results list.
    */
   public boolean _isOffEnd();

   /**
    * Get the off-end status of this query, which indicates whether the query
    * has run off either end of its results.
    *
    * @return  One of the {@link OffEnd} constants <code>NONE</code>,
    *          <code>FRONT</code>, or <code>BACK</code>.
    */
   public OffEnd getOffEnd();

   /**
    * Indicate whether this query preselects all of its results.
    *
    * @return  <code>true</code> if the query preselects its results;
    *          <code>false</code> if the query's results are dynamically
    *          navigated.
    */
   public boolean isPreselect();
   
   /**
    * Is this query set to fetch the next record after a reposition?
    *
    * @return  <code>true</code> if set to fetch after reposition, else
    *          <code>false</code>.
    */
   public boolean isFetchOnReposition();
   
   /**
    * Set this query to automatically fetch the next record after a
    * reposition, or turn this feature off.
    *
    * @param   enable
    *          <code>true</code> to enable automatic fetching;
    *          <code>false</code> to disable.
    */
   public void setFetchOnReposition(boolean enable);
   
   /**
    * Register a listener to receive notifications reposition events for this
    * query.
    *
    * @param   listener
    *          Reposition listener to be registered.
    */
   public void addRepositionListener(RepositionListener listener);
   
   /**
    * Notify all registered listeners that query is repositioned or closed.
    * 
    * @param   closed
    *          A <code>true</code> value will inform all listeners that the
    *          query was explicitly closed.
    */
   public void notifyRepositionListeners(boolean closed);
   
   /**
    * Indicate to this query that whether it is associated with a browse
    * widget or not.
    *
    * @param   browsed
    *          <code>true</code> to indicate the query has an associated
    *          browse widget, else <code>false</code>.
    */
   public void setBrowsed(boolean browsed);

   /**
    * Indicate whether this query stands alone, or is the component of
    * another query, or is wrapped by a query wrapper.
    * 
    * @return  <code>true</code> if standalone, else <code>false</code>.
    */
   public boolean isStandalone();
   
   /**
    * Change the standalone nature of this query.  A query is standalone by
    * default, but can be changed to non-standalone by the wrapper or
    * container which contains it.
    * 
    * @param   standalone
    *          <code>true</code> to set query as standalone;
    *          <code>false</code> to set query as contained.
    */
   public void setStandalone(boolean standalone);
   
   /**
    * Clean up resources associated with this query when its scope ends.
    */
   public void cleanup();

   /**
    * Conversion of GET-BUFFER-HANDLE() method (KW_GET_BUFH).
    *
    * Returns a handle of a particular buffer of a query object.
    *
    * @param   bufferName
    *          buffer-name. A CHARACTER expression that evaluates to the name
    *          of a buffer in the query or ProDataSet object.
    *
    * @return buffer handle
    */
   public handle bufferHandle(String bufferName);

   /**
    * Conversion of GET-BUFFER-HANDLE() method (KW_GET_BUFH).
    *
    * Returns a handle of a particular buffer of a query object.
    *
    * @param   bufferSequenceNumber
    *          buffer-sequence-number. An INTEGER that represents the sequence
    *          number of the desired buffer.
    *
    * @return buffer handle
    */
   public handle bufferHandle(int bufferSequenceNumber);

   /**
    * Conversion of GET-BUFFER-HANDLE() method (KW_GET_BUFH).
    *
    * Returns a handle of a particular buffer of a query object.
    *
    * @param   bufferSequenceNumber
    *          buffer-sequence-number. An INTEGER that represents the sequence
    *          number of the desired buffer.
    *
    * @return buffer handle
    */
   public handle bufferHandle(integer bufferSequenceNumber);

   /**
    * Conversion of QUERY-CLOSE() method (KW_QRY_CLOS).
    *
    * Closes a query object.
    *
    * @return  <code>true</code> if QUERY-CLOSE() succeeds,
    *          else <code>false</code>.
    */
   public logical queryClose();

   /**
    * Conversion of REPOSITION-FORWARD() method (KW_REPOS_F).
    *
    * Moves a query object’s result list pointer forward a particular number
    * of rows.
    *
    * @return  <code>true</code> if REPOSITION-FORWARD() succeeds,
    *          else <code>false</code>.
    */
   public logical queryForward(int rows);

   /**
    * Conversion of REPOSITION-TO-ROW() method (KW_REPOS_2R).
    *
    * Reposition the cursor such that a request to retrieve the next result
    * will retrieve the result which matches the specified primary key ID.  A
    * request to retrieve the previous result will retrieve the result
    * immediately previous to this, if any, in the results list.
    *
    * @param   id
    *          A primary key ID, representing the target record (in the case
    *          of a join, this ID is associated with the left-most record in
    *          the join).
    *
    * @return  <code>true</code> if REPOSITION-TO-ROW() succeeds,
    *          else <code>false</code>.
    */
   public logical queryReposition(rowid id);

   /**
    * Conversion of REPOSITION-TO-ROWID() method (KW_REPOS_2I).
    *
    * Reposition the cursor such that a request to retrieve the next result
    * will retrieve the result which matches the specified array of primary
    * key IDs.  A request to retrieve the previous result will retrieve the
    * result immediately previous to this, if any, in the results list.
    *
    * @param   id1
    *          A primary key ID, representing the target record (in the case
    *          of a join, this ID is associated with the left-most record in
    *          the join).
    * @param   joinIDs
    *          All remaining primary key IDs, if any, arranged from left to
    *          right to coincide with the records being joined by the
    *          underlying query.
    *
    * @return  <code>true</code> if REPOSITION-TO-ROWID() succeeds,
    *          else <code>false</code>.
    */
   public logical queryRepositionByID(rowid id1, rowid...joinIDs);

   /**
    * Conversion of NUM-RESULTS attribute (KW_NUM_RES).
    *
    * Getter of NUM-RESULTS attribute
    *
    */
   public integer getNumResults();

   /**
    * Adds one new buffer to a query object or dynamic ProDataSet object,
    * without affecting the other buffers, if any. Use the SET-BUFFERS( )
    * method to remove all prior buffers and set all buffers for the object
    * at the same time.
    *
    * @param   bufHandle
    *          A buffer handle to add.
    *
    * @return  true on success
    */
   public logical addBuffer(handle bufHandle);

   /**
    * Adds one new buffer to a query object or dynamic ProDataSet object, 
    * without affecting the other buffers, if any. Use the SET-BUFFERS( )
    * method to remove all prior buffers and set all buffers for the object 
    * at the same time.
    * 
    * @param   buf
    *          A buffer to add.
    *
    * @return  true on success
    */
   public logical addBuffer(Buffer buf);

   /**
    * Sets buffers for the query. Any buffers previously added
    * or set are removed. Use the <code>addBuffer()</code> method to add one 
    * buffer to the object, without affecting the other buffers, if any.
    * This method is the equivalent of <code>SET-BUFFERS()</code> method 
    * from 4GL.
    *
    * @param   hBuffers
    *          Variable argument list of buffer handles to be set.
    *          In 4GL, the maximum number of buffers per query is 18.
    * 
    * @return  true on success
    */
   public logical setBuffers(handle ... hBuffers);

   /**
    * Sets buffers for the query. Any buffers previously added
    * or set are removed. Use the <code>addBuffer()</code> method to add one 
    * buffer to the object, without affecting the other buffers, if any.
    * This method is the equivalent of <code>SET-BUFFERS()</code> method 
    * from 4GL.
    * 
    * @param   bufs
    *          Variable argument list of Buffer to be set.
    *          In 4GL, the maximum number of buffers per query is 18.
    * 
    * @return  true on success
    */
   public logical setBuffers(Buffer ... bufs);

   /**
    * Compiles a predicate (query condition). This method is
    * the equivalent of <code>QUERY-PREPARE()</code> method from 4GL.
    * 
    * @param   predicate
    *          The predicate to be prepared by this query.
    * 
    * @return  true on success
    */
   public logical prepare(String predicate);

   /**
    * Compiles a predicate (query condition). This method is
    * the equivalent of <code>QUERY-PREPARE()</code> method from 4GL.
    * 
    * @param   predicate
    *          The predicate to be prepared by this query.
    * 
    * @return  true on success
    */
   public logical prepare(character predicate);

   /**
    * Opens a query object. 
    * This method is the equivalent of <code>QUERY-OPEN()</code> method from 4GL.
    * <u>Note</u>: You must perform <code>prepare()</code> on a query 
    * object before you perform <code>queryOpen<code> on it. 
    * 
    * @return  true on success
    */
   public logical queryOpen();

   /**
    * Conversion of INDEX-INFORMATION attribute (KW_IDX_INFO).
    *
    * Getter of INDEX-INFORMATION attribute
    *
    * @return A character string consisting of a comma-separated list of the index or
    * indexes the query uses at the level of join specified.
    */
   public character indexInformation();

   /**
    * Conversion of INDEX-INFORMATION attribute (KW_IDX_INFO).
    *
    * Getter of INDEX-INFORMATION attribute
    *
    * @param n
    * An integer expression that evaluates to the level of join for which you want
    * index information.

    * @return A character string consisting of a comma-separated list of the index or
    * indexes the query uses at the level of join specified.
    */
   public character indexInformation(int n);

   /**
    * Conversion of FORWARD-ONLY attribute (KW_FWD_ONLY).
    *
    * Getter of FORWARD-ONLY attribute.
    *
    * @return current value of FORWARD-ONLY attribute.
    */
   public logical forwardOnly();

   /**
    * Conversion of FORWARD-ONLY attribute (KW_FWD_ONLY).
    *
    * Setter of FORWARD-ONLY attribute.
    *
    * @param  forwardOnly
    *         New value of FORWARD-ONLY attribute.
    */
   public void changeForwardOnly(logical forwardOnly);

   /**
    * Conversion of PREPARE_STRING attribute (KW_PREP_STR).
    *
    * Getter of PREPARE_STRING attribute.
    *
    * @return current value of PREPARE_STRING attribute.
    */
   public character prepareString();

   /**
    * Conversion of IS-OPEN attribute (KW_IS_OPEN).
    *
    * Getter of IS-OPEN attribute.
    *
    * @return current value of IS-OPEN attribute.
    */
   public logical isOpen();

   /**
    * Conversion of GET-NEXT method (KW_GET_NEXT).
    *
    * Moves a query object’s result list pointer ahead one row.
    *
    * @return  <code>true</code> if the next record in the query is found. If the query is not
    *          open or the next record cannot be found (query is empty or the query result list
    *          pointer is on the last row), the method returns <code>false</code>. If the query
    *          object handle is invalid, the method returns the Unknown value (?).
    */
   public logical getNext();

   /**
    * Conversion of GET-NEXT method (KW_GET_NEXT).
    *
    * Moves a query object’s result list pointer ahead one row.
    *
    * @param   lockType
    *          SHARE-LOCK|EXCLUSIVE-LOCK|NO-LOCK
    *
    * @return  <code>true</code> if the next record in the query is found. If the query is not
    *          open or the next record cannot be found (query is empty or the query result list
    *          pointer is on the last row), the method returns <code>false</code>. If the query
    *          object handle is invalid, the method returns the Unknown value (?).
    */
   public logical getNext(LockType lockType);

   /**
    * Conversion of GET-NEXT method (KW_GET_NEXT).
    *
    * Moves a query object’s result list pointer ahead one row.
    *
    * @param   lock
    *          SHARE-LOCK|EXCLUSIVE-LOCK|NO-LOCK
    *
    * @return  <code>true</code> if the next record in the query is found. If the query is not
    *          open or the next record cannot be found (query is empty or the query result list
    *          pointer is on the last row), the method returns <code>false</code>. If the query
    *          object handle is invalid, the method returns the Unknown value (?).
    *
    * @throws  IllegalArgumentException
    *          if <code>lock</code> has not valid internal value.
    */
   public logical getNext(long lock);

   /**
    * Conversion of GET-NEXT method (KW_GET_NEXT).
    *
    * Moves a query object’s result list pointer ahead one row.
    *
    * @param   lock
    *          SHARE-LOCK|EXCLUSIVE-LOCK|NO-LOCK
    *
    * @return  <code>true</code> if the next record in the query is found. If the query is not
    *          open or the next record cannot be found (query is empty or the query result list
    *          pointer is on the last row), the method returns <code>false</code>. If the query
    *          object handle is invalid, the method returns the Unknown value (?).
    *
    * @throws  IllegalArgumentException
    *          if <code>lock</code> has not valid internal value.
    */
   public logical getNext(NumberType lock);

   /**
    * Conversion of GET-NEXT method (KW_GET_NEXT).
    *
    * Moves a query object’s result list pointer ahead one row.
    *
    * @param   lock
    *          SHARE-LOCK|EXCLUSIVE-LOCK|NO-LOCK
    * @param   nowait
    *          NO-WAIT
    *
    * @return  <code>true</code> if the next record in the query is found. If the query is not
    *          open or the next record cannot be found (query is empty or the query result list
    *          pointer is on the last row), the method returns <code>false</code>. If the query
    *          object handle is invalid, the method returns the Unknown value (?).
    *
    * @throws  IllegalArgumentException
    *          if <code>lock</code> or <code>nowait</code> have not valid internal values.
    */
   public logical getNext(long lock, long nowait);

   /**
    * Conversion of GET-NEXT method (KW_GET_NEXT).
    *
    * Moves a query object’s result list pointer ahead one row.
    *
    * @param   lock
    *          SHARE-LOCK|EXCLUSIVE-LOCK|NO-LOCK
    * @param   nowait
    *          NO-WAIT
    *
    * @return  <code>true</code> if the next record in the query is found. If the query is not
    *          open or the next record cannot be found (query is empty or the query result list
    *          pointer is on the last row), the method returns <code>false</code>. If the query
    *          object handle is invalid, the method returns the Unknown value (?).
    *
    * @throws  IllegalArgumentException
    *          if <code>lock</code> or <code>nowait</code> have not valid internal values.
    */
   public logical getNext(long lock, NumberType nowait);

   /**
    * Conversion of GET-NEXT method (KW_GET_NEXT).
    *
    * Moves a query object’s result list pointer ahead one row.
    *
    * @param   lock
    *          SHARE-LOCK|EXCLUSIVE-LOCK|NO-LOCK
    * @param   nowait
    *          NO-WAIT
    *
    * @return  <code>true</code> if the next record in the query is found. If the query is not
    *          open or the next record cannot be found (query is empty or the query result list
    *          pointer is on the last row), the method returns <code>false</code>. If the query
    *          object handle is invalid, the method returns the Unknown value (?).
    *
    * @throws  IllegalArgumentException
    *          if <code>lock</code> or <code>nowait</code> have not valid internal values.
    */
   public logical getNext(NumberType lock, long nowait);

   /**
    * Conversion of GET-NEXT method (KW_GET_NEXT).
    *
    * Moves a query object’s result list pointer ahead one row.
    *
    * @param   lock
    *          SHARE-LOCK|EXCLUSIVE-LOCK|NO-LOCK
    * @param   nowait
    *          NO-WAIT
    *
    * @return  <code>true</code> if the next record in the query is found. If the query is not
    *          open or the next record cannot be found (query is empty or the query result list
    *          pointer is on the last row), the method returns <code>false</code>. If the query
    *          object handle is invalid, the method returns the Unknown value (?).
    *
    * @throws  IllegalArgumentException
    *          if <code>lock</code> or <code>nowait</code> have not valid internal values.
    */
   public logical getNext(NumberType lock, NumberType nowait);

   /**
    * Conversion of GET-PREV method (KW_GET_PREV).
    *
    * Moves a query object’s result list pointer back one row.
    *
    * @return  <code>true</code> if the previous record in the query is found. If the query is
    *          not open or the previous record cannot be found (query is empty or the query
    *          result list pointer is on the first row), the method returns <code>false</code>.
    *          If the query object handle is invalid, the method returns the Unknown value (?).
    */
   public logical getPrevious();

   /**
    * Conversion of GET-PREV method (KW_GET_PREV).
    *
    * Moves a query object’s result list pointer back one row.
    *
    * @param   lock
    *          SHARE-LOCK|EXCLUSIVE-LOCK|NO-LOCK
    *
    * @return  <code>true</code> if the previous record in the query is found. If the query is
    *          not open or the previous record cannot be found (query is empty or the query
    *          result list pointer is on the first row), the method returns <code>false</code>.
    *          If the query object handle is invalid, the method returns the Unknown value (?).
    *
    * @throws  IllegalArgumentException
    *          if <code>lock</code> has not valid internal value.
    */
   public logical getPrevious(long lock);

   /**
    * Conversion of GET-PREV method (KW_GET_PREV).
    *
    * Moves a query object’s result list pointer back one row.
    *
    * @param   lockType
    *          SHARE-LOCK|EXCLUSIVE-LOCK|NO-LOCK
    *
    * @return  <code>true</code> if the previous record in the query is found. If the query is
    *          not open or the previous record cannot be found (query is empty or the query
    *          result list pointer is on the first row), the method returns <code>false</code>.
    *          If the query object handle is invalid, the method returns the Unknown value (?).
    *
    * @throws  IllegalArgumentException
    *          if <code>lock</code> has not valid internal value.
    */
   public logical getPrevious(LockType lockType);

   /**
    * Conversion of GET-PREV method (KW_GET_PREV).
    *
    * Moves a query object’s result list pointer back one row.
    *
    * @param   lock
    *          SHARE-LOCK|EXCLUSIVE-LOCK|NO-LOCK
    *
    * @return  <code>true</code> if the previous record in the query is found. If the query is
    *          not open or the previous record cannot be found (query is empty or the query
    *          result list pointer is on the first row), the method returns <code>false</code>.
    *          If the query object handle is invalid, the method returns the Unknown value (?).
    *
    * @throws  IllegalArgumentException
    *          if <code>lock</code> has not valid internal value.
    */
   public logical getPrevious(NumberType lock);

   /**
    * Conversion of GET-PREV method (KW_GET_PREV).
    *
    * Moves a query object’s result list pointer back one row.
    *
    * @param   lock
    *          SHARE-LOCK|EXCLUSIVE-LOCK|NO-LOCK
    * @param   nowait
    *          NO-WAIT
    *
    * @return  <code>true</code> if the previous record in the query is found. If the query is
    *          not open or the previous record cannot be found (query is empty or the query
    *          result list pointer is on the first row), the method returns <code>false</code>.
    *          If the query object handle is invalid, the method returns the Unknown value (?).
    *
    * @throws  IllegalArgumentException
    *          if <code>lock</code> or <code>nowait</code> have not valid internal values.
    */
   public logical getPrevious(long lock, long nowait);

   /**
    * Conversion of GET-PREV method (KW_GET_PREV).
    *
    * Moves a query object’s result list pointer back one row.
    *
    * @param   lock
    *          SHARE-LOCK|EXCLUSIVE-LOCK|NO-LOCK
    * @param   nowait
    *          NO-WAIT
    *
    * @return  <code>true</code> if the previous record in the query is found. If the query is
    *          not open or the previous record cannot be found (query is empty or the query
    *          result list pointer is on the first row), the method returns <code>false</code>.
    *          If the query object handle is invalid, the method returns the Unknown value (?).
    *
    * @throws  IllegalArgumentException
    *          if <code>lock</code> or <code>nowait</code> have not valid internal values.
    */
   public logical getPrevious(NumberType lock, long nowait);

   /**
    * Conversion of GET-PREV method (KW_GET_PREV).
    *
    * Moves a query object’s result list pointer back one row.
    *
    * @param   lock
    *          SHARE-LOCK|EXCLUSIVE-LOCK|NO-LOCK
    * @param   nowait
    *          NO-WAIT
    *
    * @return  <code>true</code> if the previous record in the query is found. If the query is
    *          not open or the previous record cannot be found (query is empty or the query
    *          result list pointer is on the first row), the method returns <code>false</code>.
    *          If the query object handle is invalid, the method returns the Unknown value (?).
    *
    * @throws  IllegalArgumentException
    *          if <code>lock</code> or <code>nowait</code> have not valid internal values.
    */
   public logical getPrevious(long lock, NumberType nowait);

   /**
    * Conversion of GET-PREV method (KW_GET_PREV).
    *
    * Moves a query object’s result list pointer back one row.
    *
    * @param   lock
    *          SHARE-LOCK|EXCLUSIVE-LOCK|NO-LOCK
    * @param   nowait
    *          NO-WAIT
    *
    * @return  <code>true</code> if the previous record in the query is found. If the query is
    *          not open or the previous record cannot be found (query is empty or the query
    *          result list pointer is on the first row), the method returns <code>false</code>.
    *          If the query object handle is invalid, the method returns the Unknown value (?).
    *
    * @throws  IllegalArgumentException
    *          if <code>lock</code> or <code>nowait</code> have not valid internal values.
    */
   public logical getPrevious(NumberType lock, NumberType nowait);

   /**
    * Conversion of GET-FIRST method (KW_GET_1ST).
    *
    * Moves a query object’s result list pointer to the first row.
    *
    * @return  <code>true</code> if the first record in the query is found. If the query is not
    *          open or the first record cannot be found (query is empty), the method returns
    *          <code>false</code>. If the query object handle is invalid, the method returns
    *          the Unknown value (?).
    */
   public logical getFirst();

   /**
    * Conversion of GET-FIRST method (KW_GET_1ST).
    *
    * Moves a query object’s result list pointer to the first row.
    *
    * @param   lock
    *          SHARE-LOCK|EXCLUSIVE-LOCK|NO-LOCK
    *
    * @return  <code>true</code> if the first record in the query is found. If the query is not
    *          open or the first record cannot be found (query is empty), the method returns
    *          <code>false</code>. If the query object handle is invalid, the method returns
    *          the Unknown value (?).
    *
    * @throws  IllegalArgumentException
    *          if <code>lock</code> has not valid internal value.
    */
   public logical getFirst(long lock);

   /**
    * Conversion of GET-FIRST method (KW_GET_1ST).
    *
    * Moves a query object’s result list pointer to the first row.
    *
    * @param   lock
    *          SHARE-LOCK|EXCLUSIVE-LOCK|NO-LOCK
    *
    * @return  <code>true</code> if the first record in the query is found. If the query is not
    *          open or the first record cannot be found (query is empty), the method returns
    *          <code>false</code>. If the query object handle is invalid, the method returns
    *          the Unknown value (?).
    *
    * @throws  IllegalArgumentException
    *          if <code>lock</code> has not valid internal value.
    */
   public logical getFirst(NumberType lock);

   /**
    * Conversion of GET-FIRST method (KW_GET_1ST).
    *
    * Moves a query object’s result list pointer to the first row.
    *
    * @param   lockType
    *          SHARE-LOCK|EXCLUSIVE-LOCK|NO-LOCK
    *
    * @return  <code>true</code> if the first record in the query is found. If the query is not
    *          open or the first record cannot be found (query is empty), the method returns
    *          <code>false</code>. If the query object handle is invalid, the method returns
    *          the Unknown value (?).
    *
    * @throws  IllegalArgumentException
    *          if <code>lock</code> has not valid internal value.
    */
   public logical getFirst(LockType lockType);

   /**
    * Conversion of GET-FIRST method (KW_GET_1ST).
    *
    * Moves a query object’s result list pointer to the first row.
    *
    * @param   lock
    *          SHARE-LOCK|EXCLUSIVE-LOCK|NO-LOCK
    * @param   nowait
    *          NO-WAIT
    *
    * @return  <code>true</code> if the first record in the query is found. If the query is not
    *          open or the first record cannot be found (query is empty), the method returns
    *          <code>false</code>. If the query object handle is invalid, the method returns
    *          the Unknown value (?).
    *
    * @throws  IllegalArgumentException
    *          if <code>lock</code> or <code>nowait</code> have not valid internal values.
    */
   public logical getFirst(long lock, long nowait);

   /**
    * Conversion of GET-FIRST method (KW_GET_1ST).
    *
    * Moves a query object’s result list pointer to the first row.
    *
    * @param   lock
    *          SHARE-LOCK|EXCLUSIVE-LOCK|NO-LOCK
    * @param   nowait
    *          NO-WAIT
    *
    * @return  <code>true</code> if the first record in the query is found. If the query is not
    *          open or the first record cannot be found (query is empty), the method returns
    *          <code>false</code>. If the query object handle is invalid, the method returns
    *          the Unknown value (?).
    *
    * @throws  IllegalArgumentException
    *          if <code>lock</code> or <code>nowait</code> have not valid internal values.
    */
   public logical getFirst(NumberType lock, long nowait);

   /**
    * Conversion of GET-FIRST method (KW_GET_1ST).
    *
    * Moves a query object’s result list pointer to the first row.
    *
    * @param   lock
    *          SHARE-LOCK|EXCLUSIVE-LOCK|NO-LOCK
    * @param   nowait
    *          NO-WAIT
    *
    * @return  <code>true</code> if the first record in the query is found. If the query is not
    *          open or the first record cannot be found (query is empty), the method returns
    *          <code>false</code>. If the query object handle is invalid, the method returns
    *          the Unknown value (?).
    *
    * @throws  IllegalArgumentException
    *          if <code>lock</code> or <code>nowait</code> have not valid internal values.
    */
   public logical getFirst(long lock, NumberType nowait);

   /**
    * Conversion of GET-FIRST method (KW_GET_1ST).
    *
    * Moves a query object’s result list pointer to the first row.
    *
    * @param   lock
    *          SHARE-LOCK|EXCLUSIVE-LOCK|NO-LOCK
    * @param   nowait
    *          NO-WAIT
    *
    * @return  <code>true</code> if the first record in the query is found. If the query is not
    *          open or the first record cannot be found (query is empty), the method returns
    *          <code>false</code>. If the query object handle is invalid, the method returns
    *          the Unknown value (?).
    *
    * @throws  IllegalArgumentException
    *          if <code>lock</code> or <code>nowait</code> have not valid internal values.
    */
   public logical getFirst(NumberType lock, NumberType nowait);

   /**
    * Conversion of GET-LAST method (KW_GET_LAST).
    *
    * Moves a query object’s result list pointer to the last row.
    *
    * @return  <code>true</code> if the last record in the query is found. If the query is not
    *          open or the last record cannot be found (query is empty), the method returns
    *          <code>false</code>. If the query object handle is invalid, the method returns
    *          the Unknown value (?).
    */
   public logical getLast();

   /**
    * Conversion of GET-LAST method (KW_GET_LAST).
    *
    * Moves a query object’s result list pointer to the last row.
    *
    * @param   lock
    *          SHARE-LOCK|EXCLUSIVE-LOCK|NO-LOCK
    *
    * @return  <code>true</code> if the last record in the query is found. If the query is not
    *          open or the last record cannot be found (query is empty), the method returns
    *          <code>false</code>. If the query object handle is invalid, the method returns
    *          the Unknown value (?).
    *
    * @throws  IllegalArgumentException
    *          if <code>lock</code> has not valid internal value.
    */
   public logical getLast(long lock);

   /**
    * Conversion of GET-LAST method (KW_GET_LAST).
    *
    * Moves a query object’s result list pointer to the last row.
    *
    * @param   lock
    *          SHARE-LOCK|EXCLUSIVE-LOCK|NO-LOCK
    *
    * @return  <code>true</code> if the last record in the query is found. If the query is not
    *          open or the last record cannot be found (query is empty), the method returns
    *          <code>false</code>. If the query object handle is invalid, the method returns
    *          the Unknown value (?).
    *
    * @throws  IllegalArgumentException
    *          if <code>lock</code> has not valid internal value.
    */
   public logical getLast(NumberType lock);

   /**
    * Conversion of GET-LAST method (KW_GET_LAST).
    *
    * Moves a query object’s result list pointer to the last row.
    *
    * @param   lockType
    *          SHARE-LOCK|EXCLUSIVE-LOCK|NO-LOCK
    *
    * @return  <code>true</code> if the last record in the query is found. If the query is not
    *          open or the last record cannot be found (query is empty), the method returns
    *          <code>false</code>. If the query object handle is invalid, the method returns
    *          the Unknown value (?).
    *
    * @throws  IllegalArgumentException
    *          if <code>lock</code> has not valid internal value.
    */
   public logical getLast(LockType lockType);

   /**
    * Conversion of GET-LAST method (KW_GET_LAST).
    *
    * Moves a query object’s result list pointer to the last row.
    *
    * @param   lock
    *          SHARE-LOCK|EXCLUSIVE-LOCK|NO-LOCK
    * @param   nowait
    *          NO-WAIT
    *
    * @return  <code>true</code> if the last record in the query is found. If the query is not
    *          open or the last record cannot be found (query is empty), the method returns
    *          <code>false</code>. If the query object handle is invalid, the method returns
    *          the Unknown value (?).
    *
    * @throws  IllegalArgumentException
    *          if <code>lock</code> or <code>nowait</code> have not valid internal values.
    */
   public logical getLast(long lock, long nowait);

   /**
    * Conversion of GET-LAST method (KW_GET_LAST).
    *
    * Moves a query object’s result list pointer to the last row.
    *
    * @param   lock
    *          SHARE-LOCK|EXCLUSIVE-LOCK|NO-LOCK
    * @param   nowait
    *          NO-WAIT
    *
    * @return  <code>true</code> if the last record in the query is found. If the query is not
    *          open or the last record cannot be found (query is empty), the method returns
    *          <code>false</code>. If the query object handle is invalid, the method returns
    *          the Unknown value (?).
    *
    * @throws  IllegalArgumentException
    *          if <code>lock</code> or <code>nowait</code> have not valid internal values.
    */
   public logical getLast(NumberType lock, long nowait);

   /**
    * Conversion of GET-LAST method (KW_GET_LAST).
    *
    * Moves a query object’s result list pointer to the last row.
    *
    * @param   lock
    *          SHARE-LOCK|EXCLUSIVE-LOCK|NO-LOCK
    * @param   nowait
    *          NO-WAIT
    *
    * @return  <code>true</code> if the last record in the query is found. If the query is not
    *          open or the last record cannot be found (query is empty), the method returns
    *          <code>false</code>. If the query object handle is invalid, the method returns
    *          the Unknown value (?).
    *
    * @throws  IllegalArgumentException
    *          if <code>lock</code> or <code>nowait</code> have not valid internal values.
    */
   public logical getLast(long lock, NumberType nowait);

   /**
    * Conversion of GET-LAST method (KW_GET_LAST).
    *
    * Moves a query object’s result list pointer to the last row.
    *
    * @param   lock
    *          SHARE-LOCK|EXCLUSIVE-LOCK|NO-LOCK
    * @param   nowait
    *          NO-WAIT
    *
    * @return  <code>true</code> if the last record in the query is found. If the query is not
    *          open or the last record cannot be found (query is empty), the method returns
    *          <code>false</code>. If the query object handle is invalid, the method returns
    *          the Unknown value (?).
    *
    * @throws  IllegalArgumentException
    *          if <code>lock</code> or <code>nowait</code> have not valid internal values.
    */
   public logical getLast(NumberType lock, NumberType nowait);

   /**
    * Conversion of GET-CURRENT() method (KW_GET_CUR).
    *
    * Refetches the current record or records associated with the query.
    *
    * @return  <code>true</code> if GET-CURRENT() succeeds, else <code>false</code>.
    */
   public logical getCurrent();

   /**
    * Conversion of GET-CURRENT() method (KW_GET_CUR).
    *
    * Refetches the current record or records associated with the query.
    *
    * @param   lock
    *          SHARE-LOCK|EXCLUSIVE-LOCK|NO-LOCK
    *
    * @return  <code>true</code> if GET-CURRENT() succeeds, else <code>false</code>.
    */
   public logical getCurrent(long lock);

   /**
    * Conversion of GET-CURRENT() method (KW_GET_CUR).
    *
    * Refetches the current record or records associated with the query.
    *
    * @param   lock
    *          SHARE-LOCK|EXCLUSIVE-LOCK|NO-LOCK
    *
    * @return  <code>true</code> if GET-CURRENT() succeeds, else <code>false</code>.
    */
   public logical getCurrent(NumberType lock);

   /**
    * Conversion of GET-CURRENT() method (KW_GET_CUR).
    *
    * Refetches the current record or records associated with the query.
    *
    * @param   lockType
    *          Lock type
    *
    * @return  <code>true</code> if GET-CURRENT() succeeds, else <code>false</code>.
    */
   public logical getCurrent(LockType lockType);

   /**
    * Conversion of GET-CURRENT() method (KW_GET_CUR).
    *
    * Refetches the current record or records associated with the query.
    *
    * @param   lock
    *          SHARE-LOCK|EXCLUSIVE-LOCK|NO-LOCK
    * @param   nowait
    *          NO-WAIT
    *
    * @return  <code>true</code> if GET-CURRENT() succeeds, else <code>false</code>.
    */
   public logical getCurrent(long lock, long nowait);

   /**
    * Conversion of GET-CURRENT() method (KW_GET_CUR).
    *
    * Refetches the current record or records associated with the query.
    *
    * @param   lock
    *          SHARE-LOCK|EXCLUSIVE-LOCK|NO-LOCK
    * @param   nowait
    *          NO-WAIT
    *
    * @return  <code>true</code> if GET-CURRENT() succeeds, else <code>false</code>.
    */
   public logical getCurrent(NumberType lock, long nowait);

   /**
    * Conversion of GET-CURRENT() method (KW_GET_CUR).
    *
    * Refetches the current record or records associated with the query.
    *
    * @param   lock
    *          SHARE-LOCK|EXCLUSIVE-LOCK|NO-LOCK
    * @param   nowait
    *          NO-WAIT
    *
    * @return  <code>true</code> if GET-CURRENT() succeeds, else <code>false</code>.
    */
   public logical getCurrent(long lock, NumberType nowait);

   /**
    * Conversion of GET-CURRENT() method (KW_GET_CUR).
    *
    * Refetches the current record or records associated with the query.
    *
    * @param   lock
    *          SHARE-LOCK|EXCLUSIVE-LOCK|NO-LOCK
    * @param   nowait
    *          NO-WAIT
    *
    * @return  <code>true</code> if GET-CURRENT() succeeds, else <code>false</code>.
    */
   public logical getCurrent(NumberType lock, NumberType nowait);
}
